<?php defined('ABSPATH') or die("No direct access allowed");
/*
* Plugin Name:   eShop Checkout Dynamic States
* Plugin URI:	 
* Description:   Dynamically load correct State/County/Province in Checkout forms according to selected country.
* Version:       1.0.1
* Author:        Vinny Alves
* Author URI:    http://www.usestrict.net
*
* License:       GNU General Public License, v2 (or newer)
* License URI:  http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* Copyright (C) 2012 www.usestrict.net, released under the GNU General Public License.
*/
define ('ESHOP_DYNAMIC_STATES_ABSPATH', plugin_dir_path(__FILE__));
define ('ESHOP_DYNAMIC_STATES_INCLUDES', ESHOP_DYNAMIC_STATES_ABSPATH . '/includes');
define ('ESHOP_DYNAMIC_STATES_DOMAIN', 'eshop-checkout-dynamic-states');
define ('ESHOP_DYNAMIC_STATES_INCLUDES_URL', plugins_url( ESHOP_DYNAMIC_STATES_DOMAIN . '/includes'));
define ('ESHOP_DYNAMIC_STATES_VERSION', '1.0.1');

class eShop_Checkout_Dynamic_States 
{
	protected $country_table;
	protected $states_table;
	protected $domain = ESHOP_DYNAMIC_STATES_DOMAIN;
	
	public function __construct()
	{
		global $wpdb;
		
		require_once(ABSPATH . 'wp-admin/includes/upgrade.php'); // Required for dbDelta
		
		$this->country_table = $wpdb->prefix . 'usc_countries';
		$this->states_table  = $wpdb->prefix . 'usc_states';
		
		// Set up data upon activation
		register_activation_hook(__FILE__ , array(&$this,'install_data'));
		
		// Add JS script if not admin, ajax stuff otherwise.
		if (! is_admin())
		{
// 			require_once( ABSPATH . 'wp-includes/pluggable.php'); // imports is_user_logged_in()
			
			wp_enqueue_script( $this->domain . '-refresh-states', ESHOP_DYNAMIC_STATES_INCLUDES_URL . '/eshop-checkout-dynamic-states.js', array( 'jquery' ),  
							  ESHOP_DYNAMIC_STATES_VERSION);
			
			// declare the URL to the file that handles the AJAX request (wp-admin/admin-ajax.php)
			wp_localize_script( $this->domain . '-refresh-states', 'eShopDynamicStates', array( 'ajaxurl'    => admin_url( 'admin-ajax.php' ),
																								'ajaxaction' => $this->domain . '-refresh-states',
																								'method'     => 'GET',
																								'includes_url' => ESHOP_DYNAMIC_STATES_INCLUDES_URL) 
			);
		}
		
		// Add ajax handling for non-logged in people
		add_action('wp_ajax_nopriv_' . $this->domain . '-refresh-states', array(&$this,'refresh_states'));
			
		// and for logged-in people
		add_action('wp_ajax_' . $this->domain . '-refresh-states', array(&$this,'refresh_states'));
	}
	
	
	/**
	 * Method: refresh_states()
	 * @param string $country_code
	 * Description: Ajax interface for refreshing the state list when country changes
	 */
	public function refresh_states()
	{
		global $wpdb;
		
		$country_code = $_REQUEST['country_code'];
		
		$sql =<<<EOF
select state_code, state_name 
from $this->states_table
where country_code = %s
EOF;

		$sql  = $wpdb->prepare($sql, $country_code);
		$rows = $wpdb->get_results($sql,ARRAY_A);
		
		$states = array();
		foreach ($rows as $row)
		{
			$name           = $row['state_name'];
			$states[$name] = $row['state_code'];
		}
		
		header("Content-type: application/json");

		$out['success'] = true;
        $out['data']    = $states;

        echo json_encode($out);
        exit; // WP requirement for ajax-related methods
	}
	
	/**
	 * @method  install_data
	 * @desc    Installs initial data
	 */
	public function install_data()
	{
		$this->_install_countries();
		$this->_install_states();
	}
	
	
	/**
	 * @method  _install_countries
	 * @desc    Installs Countries
	 */
	private function _install_countries()
	{
		global $wpdb;

		$sql =<<<EOF
CREATE TABLE IF NOT EXISTS $this->country_table (
  country_code char(2) NOT NULL,
  country_name varchar(100) NOT NULL,
  PRIMARY KEY  (country_code)
) ENGINE=InnoDB;
EOF;
		$out = dbDelta($sql);
		
		if (FALSE !== ($fh = gzopen(ESHOP_DYNAMIC_STATES_INCLUDES . '/data/countries.csv.gz', 'rb')))
		{
			while (FALSE !== ($row = fgetcsv($fh,500)))
			{
				$wpdb->insert($this->country_table,array('country_code' => $row[0], 'country_name' => $row[1]));
			}	
		}
	}
	
	
	/**
	 * @method  _install_states
	 * @desc    Installs States
	 */
	private function _install_states()
	{
		global $wpdb;
		
		$sql =<<<EOF
CREATE TABLE IF NOT EXISTS $this->states_table(
  state_code varchar(50) NOT NULL,
  state_name varchar(100) NOT NULL,
  country_code char(2) NOT NULL,
  PRIMARY KEY  (state_code),
  CONSTRAINT usc_country_code_fk2 FOREIGN KEY  (country_code) REFERENCES $this->country_table (country_code) ON UPDATE CASCADE ON DELETE CASCADE
) ENGINE=InnoDB;
EOF;
		$out = dbDelta($sql);
		
		// Alter the size of the table if it already existed
		$sql =<<<EOF
ALTER TABLE $this->states_table
CHANGE COLUMN state_code state_code varchar(50) NOT NULL;
EOF;
		
		$out = dbDelta($sql);
		
		if (FALSE !== ($fh = gzopen(ESHOP_DYNAMIC_STATES_INCLUDES . '/data/states.csv.gz', 'rb')))
		{
			while (FALSE !== ($row = fgetcsv($fh,500)))
			{
				$wpdb->insert($this->states_table,array('state_code' => $row[0], 'state_name' => $row[1], 'country_code' => $row[2]));
			}
		}
		
	}
}

$eShop_Checkout_Dynamic_States = new eShop_Checkout_Dynamic_States();


/* End of file eshop-checkout-dynamic-states.php */
/* Location: eshop-checkout-dynamic-states/eshop-checkout-dynamic-states.php */